// Copyright (c) 2021 Visiosto oy
// Licensed under the MIT License

import React from 'react';
import Terser from 'terser';
import PropTypes from 'prop-types';
import {
  colorModeKey as defaultColorModeKey,
  initialColorModeCSSProp,
} from '@visiosto/react-components';

// Thanks to Joshua Comeau for the original code, licensed under MIT License:
// https://github.com/joshwcomeau/dark-mode-minimal

// Don't use named function expression for this function just in case - this is embedded to the site
function setColorsByTheme() {
  const colors = '🌈';
  const colorModeKey = '🔑';
  const colorModeCssProp = '⚡️';

  const mediaQuery = window.matchMedia('(prefers-color-scheme: dark)');
  const prefersDarkScheme = mediaQuery.matches;
  const persistedPreference = localStorage.getItem(colorModeKey);

  let colorMode = 'light';

  // TODO Uncomment the line if a dark mode toggle is added
  // const hasUsedToggle = typeof persistedPreference === 'string';
  const hasUsedToggle = false;

  if (hasUsedToggle) {
    colorMode = persistedPreference;
  } else {
    colorMode = prefersDarkScheme ? 'dark' : 'light';
  }

  const root = document.documentElement;
  const invertedColorMode = colorMode === 'dark' ? 'light' : 'dark';

  root.style.setProperty(colorModeCssProp, colorMode);

  Object.entries(colors).forEach(([name, colorByTheme]) => {
    const cssVarName = `--color-${name}`;

    root.style.setProperty(cssVarName, colorByTheme[colorMode]);
    root.style.setProperty(`${cssVarName}-inverted`, colorByTheme[invertedColorMode]);
  });
}

// eslint-disable-next-line react/forbid-prop-types
const magicScriptTagPropTypes = { colors: PropTypes.object.isRequired };

function MagicScriptTag({ colors }) {
  const boundFn = String(setColorsByTheme)
    .replace("'🌈'", JSON.stringify(colors))
    .replace('🔑', defaultColorModeKey)
    .replace('⚡️', initialColorModeCSSProp);

  let calledFunction = `(${boundFn})()`;

  calledFunction = Terser.minify(calledFunction).code;

  // eslint-disable-next-line react/no-danger, react/jsx-filename-extension
  return <script dangerouslySetInnerHTML={{ __html: calledFunction }} />;
}

MagicScriptTag.propTypes = magicScriptTagPropTypes;

// eslint-disable-next-line react/forbid-prop-types
const fallbackStylesPropTypes = { colors: PropTypes.object.isRequired };

/**
 * If the user has JS disabled, the injected script will never fire!
 * This means that they won't have any colors set, everything will be default
 * black and white.
 * We can solve for this by injecting a `<style>` tag into the head of the
 * document, which sets default values for all of our colors.
 * Only light mode will be available for users with JS disabled.
 */
function FallbackStyles({ colors }) {
  // Create a string holding each CSS variable:
  /*
    `--color-text: black;
    --color-background: white;`
  */

  const cssVariableString = Object.entries(colors).reduce(
    (acc, [name, colorByTheme]) =>
      `${acc}\n--color-${name}: ${colorByTheme.light};\n--color-${name}-inverted: ${colorByTheme.dark};`,
    '',
  );

  const wrappedInSelector = `html { ${cssVariableString} }`;

  // eslint-disable-next-line react/jsx-filename-extension
  return <style>{wrappedInSelector}</style>;
}

FallbackStyles.propTypes = fallbackStylesPropTypes;

// eslint-disable-next-line import/prefer-default-export
export const onRenderBody = function addColorThemeStylesOnRenderBody(
  { setPreBodyComponents, setHeadComponents },
  { colors },
) {
  setHeadComponents(<FallbackStyles key="color-scheme-fallback-styles" colors={colors} />);
  setPreBodyComponents(<MagicScriptTag key="color-scheme-magic-script-tag" colors={colors} />);
};
